# -*- encoding: UTF-8 -*-
# (c) 2011 Michał Węgrzynek, Litex
# for license see http://repoze.org/license.html
'''
A SqlAlchemy connection pool utilizing cx_Oracle's SessionPool object.
It's intended to be created independently and passed by pool keyword
argument to create_engine.
'''
import cx_Oracle as cx
from sqlalchemy.pool import NullPool
from sqlalchemy.engine.url import make_url


class CxOracleSessionPool(NullPool):
    
    _v_session_pool = None
    
    def __init__(
        self, url_string, min_sessions=5, max_sessions=20, increment=2,
        user_source=None, **kw
    ):
        self.url_string = url_string
        self.min_sessions = min_sessions
        self.max_sessions = max_sessions
        self.increment = increment
        
        if user_source is None:
            self.user_source = lambda: None
        else:
            self.user_source = user_source
                
        super(CxOracleSessionPool, self).__init__(self._connect, **kw)
    
    @property
    def session_pool(self):
        if not self._v_session_pool:
            url = make_url(self.url_string)            
            
            self._v_session_pool = cx.SessionPool(
                url.username,
                url.password,
                cx.makedsn(url.host, url.port if url.port else 1521, url.database),
                self.min_sessions,
                self.max_sessions,
                self.increment,
                threaded=True,
                homogeneous=False
            )
            
        return self._v_session_pool
    
    def _connect(self):                    
        username = self.user_source()
        if username:
            return self.session_pool.acquire(username)
        else:
            return self.session_pool.acquire()
        
    def do_return_conn(self, conn):
        self.session_pool.release(conn.connection)
        conn.close()     
        
    def recreate(self):    
        self.logger.info("Pool recreating")

        return CxOracleSessionPool(
            self.url_string, 
            self.min_sessions, 
            self.max_sessions, 
            self.increment,
            self.user_source,
            recycle=self._recycle,
            echo=self.echo,
            logging_name=self._orig_logging_name,
            use_threadlocal=self._use_threadlocal,
            listeners=self.listeners
        )

    def dispose(self):
        self._v_session_pool = None
        self.logger.info('Pool disposed')
        
    def status(self):
        return 'CxOracleSessionPool id=%s max sessions=%d current session count=%d' % (
            id(self),
            self.max_sessions,
            self.session_pool.busy
        )
    
    
  


# -*- encoding: UTF-8 -*-
import os
import sys

from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.txt')).read()
CHANGES = open(os.path.join(here, 'CHANGES.txt')).read()

requires = [
    'cx_Oracle',
    'sqlalchemy'
]

setup(
    name='litex.cxpool',
    version='1.0',
    description='Native Oracle Session Pool implementation for SQLAlchemy',
    long_description=README + '\n\n' +  CHANGES,
    classifiers=[
        "Programming Language :: Python",
        "Topic :: Database :: Front-Ends"
    ],
    author='Michal Wegrzynek',
    author_email='mwegrzynek@litex.pl',
    url='https://code.launchpad.net/litex.cxpool',
    license='BSD like, see http://repoze.org/license.html',
    keywords='cxpool oracle sessionpool sqlalchemy proxy',
    package_dir={'': 'src'},
    packages=find_packages('src'),
    include_package_data=True,
    zip_safe=False,    
    test_suite='nose.collector',
    install_requires = requires,
    tests_require=['nose']
)



# -*- encoding: UTF-8 -*-
import os


from nose.tools import eq_, ok_, assert_raises
from cx_Oracle import DatabaseError
from sqlalchemy import create_engine


DSN = os.environ.get('CX_DSN', 'oracle://test_user:test_password@test_server/test')
CONNECT_USER = os.environ.get('CX_CONNECT_USER', 'TEST_USER')
PROXY_USER = os.environ.get('CX_PROXY_USER', 'PROXY_USER')


def test_creation():
    from litex.cxpool import CxOracleSessionPool
    pool = CxOracleSessionPool(DSN, 1, 5, 1)
    
    new_pool = pool.recreate()
    eq_(pool.url_string, new_pool.url_string)
    eq_(pool.min_sessions, new_pool.min_sessions)
    eq_(pool.max_sessions, new_pool.max_sessions)
    eq_(pool.increment, new_pool.increment)
    
    
def test_checkout_and_status():
    from litex.cxpool import CxOracleSessionPool    
    
    pool = CxOracleSessionPool(DSN, 1, 3, 1)
    engine = create_engine('oracle://', pool=pool)
    
    conn_1 = engine.connect()
    eq_(pool.session_pool.busy, 1)
    
    conn_2 = engine.connect()
    eq_(pool.session_pool.busy, 2)
    
    conn_3 = engine.connect()
    eq_(pool.session_pool.busy, 3)
    
    assert_raises(DatabaseError, lambda: engine.connect())
    eq_(pool.session_pool.busy, 3)
    
    status = pool.status()
    ok_('CxOracleSessionPool' in status)
    ok_('max sessions=3' in status)
    ok_('current session count=3' in status)
    
    del conn_3
    eq_(pool.session_pool.busy, 2)


def test_proxy_auth():
    from litex.cxpool import CxOracleSessionPool    
    def user_src():
        return PROXY_USER
    
    pool = CxOracleSessionPool(DSN, 1, 3, 1)
    engine = create_engine('oracle://', pool=pool)
    
    conn_1 = engine.connect()
    res = conn_1.execute('SELECT USER FROM DUAL')
    
    usr = res.first()['user']
    eq_(usr, CONNECT_USER)
    
    pool = CxOracleSessionPool(DSN, 1, 3, 1, user_source=user_src)
    engine = create_engine('oracle://', pool=pool)
    
    conn_1 = engine.connect()
    res = conn_1.execute('SELECT USER FROM DUAL')
    
    usr = res.first()['user']
    eq_(usr, PROXY_USER)
    

def test_dispose():
    from litex.cxpool import CxOracleSessionPool
    pool = CxOracleSessionPool(DSN, 1, 3, 1)
    
    pool.dispose()